#include <Adafruit_NeoPixel.h>

#define PIN 3
#define LAMPS 20

// adafruit init for this particular strip (see adafruit.com)
Adafruit_NeoPixel strip = Adafruit_NeoPixel(LAMPS, PIN, NEO_GRBW + NEO_KHZ800);

uint32_t pacman = strip.Color(230,180, 0,0);
uint32_t clyde = strip.Color(190,70,0,25);
uint32_t blinky = strip.Color(219, 0, 0,10);
uint32_t pinky = strip.Color(220, 90, 176,0);
uint32_t inky = strip.Color(80, 180, 255,0);
uint32_t scaredghost = strip.Color(0,0,255,0);
uint32_t off = strip.Color(0, 0, 0,0);

// we are lighting max 15 leds at the same time, setting the max brightness lower
// allows us to light all the leds from an arduino board without separate power source
// (15 leds * 33mA = 495 mA)
void setup() {
  strip.begin();
  strip.setBrightness(255);    // overall brightness can be 0 to 255
  strip.show();               // Initialize all pixels to 'off'
}

void loop() {
  // Normal lamp, white 60 sec
  setLamp();
  
  // white food pixels
//  setFood();
  delay(1500);
  
  // pacman eats the food
  eatFood();
  delay(1500);

  // pacman is chased by ghosts
  chasedByGhosts();
  delay(1500);

  // now pacman is chasing the ghosts
  chaseGhosts();
  delay(1500);
}

//
void setLamp(){
  // fade in
  for(int br=0; br<=255; br++) {
    uint32_t white = strip.Color(0, 0, 0, br);
    for(int i=0; i<strip.numPixels(); i++) {
        strip.setPixelColor(i,white); 
    }
    strip.show();
    delay(50);
  }

  for(int i = 0; i < LAMPS; i++){
    strip.setPixelColor(i,0, 0, 0,255); 
  }
  strip.show();
  delay(600000);
// fade out
    for(int br=255; br>=0; br--) {
    uint32_t white = strip.Color(0,0,0,br);
    for(int i=0; i<strip.numPixels(); i++) {
        strip.setPixelColor(i,white);
        i++;
        strip.setPixelColor(i,white);
        i++;
    }
    strip.show();
    delay(15);
  }
}

// This isn't used
void setFood() {
// fade in
  for(int br=0; br<255; br++) {
    uint32_t white = strip.Color(0, 0, 0, br);
    for(int i=0; i<strip.numPixels(); i+=3) {
        strip.setPixelColor(i,white); 
    }
    strip.show();
    delay(5);
  }
}

void eatFood(){
  for(uint16_t i=0; i<strip.numPixels(); i++) {
      strip.setPixelColor(i,230,180,0,0);
      if(i>0){
        strip.setPixelColor(i-1,off);
      }
      strip.show();
      delay(80);
  }
  strip.setPixelColor(strip.numPixels()-1,off);
  strip.show();
  delay(80);
}

void chasedByGhosts() {
  // run back
  for(int i=strip.numPixels(); i>-30; i--) {
      // check: color, position, space between pacman and ghost
      checkLed(pacman, i, 0);
      checkLed(clyde, i, 5);
      checkLed(blinky, i, 8);
      checkLed(pinky, i, 11);
      checkLed(inky, i, 14);
      strip.show();
      delay(80);
  }
}

void chaseGhosts() {
  // run ghost, run!
  int maximum = strip.numPixels() + 5;
  for(int j = -30; j<maximum; j++) {
      checkLed(scaredghost, j, 14);
      checkLed(scaredghost, j, 11);
      checkLed(scaredghost, j, 8);
      checkLed(scaredghost, j, 5);
      checkLed(pacman, j, 0);
      strip.show();
      delay(80);
  }
}

// check if this position is within the led strip, if yes then show the color
void checkLed(uint32_t color, int i, int offset) {
  int pos = i + offset;

  if(pos < strip.numPixels() && pos > -1){
     strip.setPixelColor(pos, color); 
  }
  // switch off the pixel before and after this one (depends on direction)
  if(pos+1 < strip.numPixels() && pos+1 > -1){
     strip.setPixelColor(pos+1, strip.Color(0, 0, 0,0));
  } 
  if(pos-1 < strip.numPixels() && pos-1 > -1){
    strip.setPixelColor(pos-1, strip.Color(0, 0, 0,0));
  } 
}
